@@ -4,7 +4,8 @@ use warnings;
 use CHI::Test;
 use CHI::Test::Util
   qw(activate_test_logger cmp_bool is_between random_string skip_until);
-use CHI::Util qw(dump_one_line);
+use CHI::Util qw(dump_one_line write_file);
+use File::Spec::Functions qw(tmpdir);
 use File::Temp qw(tempdir);
 use List::Util qw(shuffle);
 use Module::Load::Conditional qw(can_load check_install);
@@ -1090,11 +1091,8 @@ sub test_logging : Test(12) {
 
     my $driver = $cache->label;
 
-    # Multilevel cache logs less details about misses
-    my $miss_not_in_cache =
-      ( $driver eq 'Multilevel' ? 'MISS' : 'MISS \(not in cache\)' );
-    my $miss_expired =
-      ( $driver eq 'Multilevel' ? 'MISS' : 'MISS \(expired\)' );
+    my $miss_not_in_cache = 'MISS \(not in cache\)';
+    my $miss_expired      = 'MISS \(expired\)';
 
     my $start_time = time();
 
@@ -1131,6 +1129,86 @@ sub test_logging : Test(12) {
     $log->empty_ok();
 }
 
+sub test_stats : Test(5) {
+    my $self = shift;
+
+    my $stats = $self->testing_chi_root_class->stats;
+    $stats->enable();
+
+    my ( $key, $value ) = $self->kvpair();
+    my $start_time = time();
+
+    my $cache;
+    $cache = $self->new_cache( namespace => 'Foo' );
+    $cache->get($key);
+    $cache->set( $key, $value, 80 );
+    $cache->get($key);
+    local $CHI::Driver::Test_Time = $start_time + 120;
+    $cache->get($key);
+    $cache->remove($key);
+    $cache->get($key);
+
+    $cache = $self->new_cache( namespace => 'Bar' );
+    $cache->set( $key, scalar( $value x 3 ) );
+    $cache->set( $key, $value );
+
+    my $log   = activate_test_logger();
+    my $label = $cache->label;
+    $log->empty_ok();
+    $stats->flush();
+    $log->contains_ok(
+        qr/CHI stats: namespace='Foo'; cache='$label'; start=.*; end=.*; absent_misses=2; expired_misses=1; hits=1; set_key_size=6; set_value_size=20; sets=1/
+    );
+    $log->contains_ok(
+        qr/CHI stats: namespace='Bar'; cache='$label'; start=.*; end=.*; set_key_size=12; set_value_size=52; sets=2/
+    );
+    $log->empty_ok();
+
+    my @logs = (
+        "CHI stats: namespace='Foo'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=3; sets=5",
+        "CHI stats: namespace='Foo'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=1; sets=7",
+        "CHI stats: namespace='Bar'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=4; sets=9",
+        "CHI stats: namespace='Foo'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; sets=3",
+        "CHI stats: namespace='Foo'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=8",
+        "CHI stats: namespace='Foo'; cache='Memory'; start=20090102:12:53:05; end=20090102:12:58:05; sets=2",
+        "CHI stats: namespace='Bar'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=10; sets=1",
+        "CHI stats: namespace='Bar'; cache='File'; start=20090102:12:53:05; end=20090102:12:58:05; hits=3; set_errors=2",
+    );
+    my $log_dir = tempdir( "chi-test-stats-XXXX", TMPDIR => 1, CLEANUP => 1 );
+    write_file( "$log_dir/log1", join( "\n", splice( @logs, 0, 4 ) ) . "\n" );
+    write_file( "$log_dir/log2", join( "\n", @logs ) );
+    open( my $fh2, "<", "$log_dir/log2" ) or die "cannot open $log_dir/log2";
+    my $results = $stats->parse_stats_logs( "$log_dir/log1", $fh2 );
+    close($fh2);
+    cmp_deeply(
+        $results,
+        [
+            {
+                root_class => 'CHI',
+                namespace  => 'Foo',
+                cache      => 'File',
+                hits       => 12,
+                sets       => 15
+            },
+            {
+                root_class => 'CHI',
+                namespace  => 'Bar',
+                cache      => 'File',
+                hits       => 17,
+                sets       => 10,
+                set_errors => 2
+            },
+            {
+                root_class => 'CHI',
+                namespace  => 'Foo',
+                cache      => 'Memory',
+                sets       => 2
+            }
+        ],
+        'parse_stats_logs'
+    );
+}
+
 sub test_cache_object : Test(6) {
     my $self  = shift;
     my $cache = $self->{cache};
@@ -1302,7 +1380,8 @@ sub test_discard_timeout : Test(4) {
     my $start_time = time;
     $cache->set( 2, 2 );
     throws_ok { $cache->discard_to_size(0) } qr/discard timeout .* reached/;
-    ok( time > $start_time && time < $start_time + 3, "timed out in 1 second" );
+    ok( time >= $start_time && time <= $start_time + 3,
+        "timed out in 1 second" );
 }
 
 sub test_size_awareness_with_subcaches : Test(19) {
@@ -1420,7 +1499,6 @@ sub test_obj_ref : Tests(8) {
     my ( $key, $value ) = ( 'medium', [ a => 5, b => 6 ] );
 
     my $validate_obj = sub {
-        my $obj = shift;
         isa_ok( $obj, 'CHI::CacheObject' );
         is( $obj->key, $key, "keys match" );
         cmp_deeply( $obj->value, $value, "values match" );
@@ -1429,11 +1507,11 @@ sub test_obj_ref : Tests(8) {
     $cache->get( $key, obj_ref => \$obj );
     ok( !defined($obj), "obj not defined on miss" );
     $cache->set( $key, $value, { obj_ref => \$obj } );
-    $validate_obj->($obj);
+    $validate_obj->();
     undef $obj;
     ok( !defined($obj), "obj not defined before get" );
     $cache->get( $key, obj_ref => \$obj );
-    $validate_obj->($obj);
+    $validate_obj->();
 }
 
 sub test_metacache : Tests(3) {
@@ -1478,8 +1556,7 @@ sub test_no_leak : Tests(2) {
     ok( !defined($weakref), "weakref is no longer defined - cache was freed" );
 }
 
-## no critic
-{ package My::CHI; use Moose; extends 'CHI' }
+Class::MOP::Class->create( 'My::CHI' => ( superclasses => ['CHI'] ) );
 
 sub test_driver_properties : Tests(2) {
     my $self  = shift;
